@@ -4,19 +4,19 @@ use strict;
 use warnings;
 use Carp;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 use base qw( SWISH::API::Stat );
 
 sub init
 {
     my $self = shift;
-    
-    $self->SUPER::init(@_); # Stat init()
-        
-    $self->mk_accessors(qw( properties class stash ));
-    
+
+    $self->SUPER::init(@_);    # Stat init()
+
+    $self->mk_accessors(qw( properties class stash serial_format ));
+
     my $i = $self->indexes->[0];    # just use the first one for header vals
-    
+
     unless ($self->properties && ref($self->properties))
     {
         $self->properties({});
@@ -28,10 +28,11 @@ sub init
         }
     }
 
+    my $d = $self->handle->header_value($i, 'Description') || '';
+    my ($class, $format) = split(m/\ +/, $d);
     unless ($self->class)
     {
-        my $d = $self->handle->header_value($i, 'Description') || '';
-        if ($d =~ m/^class:(\S+)/)
+        if ($class && $class =~ m/class:(\S+)/)
         {
             $self->class($1);
         }
@@ -40,6 +41,17 @@ sub init
             $self->class('SWISH::API::Object::Result::Instance');
         }
     }
+    unless ($self->serial_format)
+    {
+        if ($format && $format =~ m/format:(\S+)/)
+        {
+            $self->serial_format($1);
+        }
+        else
+        {
+            $self->serial_format('yaml');
+        }
+    }
 
     # this ISA trickery has 2 benefits:
     # (1) a default new() method
@@ -63,17 +75,18 @@ sub props
 
 1;
 
-
 package SWISH::API::Object::Results;
 use strict;
 use warnings;
 use base qw( SWISH::API::More::Results );
 use Carp;
+use YAML::Syck ();
+use JSON::Syck ();
 
 sub next_result
 {
     my $self = shift;
-    my $r = $self->SUPER::next_result(@_);
+    my $r    = $self->SUPER::next_result(@_);
     return undef unless defined $r;
     return $self->make_object($r);
 }
@@ -96,21 +109,9 @@ sub make_object
         }
         else
         {
-
-# TODO there must be a bug here somewhere.
-# first time this runs under persistent process, fine.
-# 2nd and subsequent, this step takes 2x as long
-# it's pretty consistent. I think it's eval that is slow
-# but why it would run 2x as fast the first time, I don't know.
-
-            # silence any eval warnings due to string content
-            no warnings 'all';
-
-            # test first if value evals to a reference,
-            # otherwise use as raw string (scalar)
-            # this adds some overhead obviously. but so does this whole module
-            my $tmp = eval $result->property($p);
-            $propvals{$p} = ref $tmp ? $tmp : $result->property($p);
+            my $v = $result->property($p);
+            $propvals{$p} =
+              defined($v) ? $self->deserialize($sao->serial_format, $v) : '';
         }
 
     }
@@ -118,6 +119,39 @@ sub make_object
     return $class->new(\%propvals, $sao->stash);
 }
 
+sub deserialize
+{
+    my $self = shift;
+    my $f    = shift;
+    my $v    = shift;
+    if ($f eq 'yaml')
+    {
+        if ($v =~ m/^---\ ([^!].*)/s)    # simple scalar
+        {
+            return $1;
+        }
+        else
+        {
+            return YAML::Syck::Load($v);
+        }
+    }
+    elsif ($f eq 'json')
+    {
+        if ($v =~ m/^[^\{\[]/)
+        {
+            return $v;
+        }
+        else
+        {
+            return JSON::Syck::Load($v);
+        }
+    }
+    else
+    {
+        croak "unknown serial format $f";
+    }
+}
+
 1;
 
 __END__
@@ -139,7 +173,8 @@ SWISH::API::Object - return SWISH::API results as objects
                         },
                     stash       => {
                                 dbh => DBI->connect($myinfo)
-                                }
+                                },
+                    serial_format => 'yaml'
                     );
                     
   my $results = $swish->query('foo');
@@ -210,6 +245,15 @@ format.
 Pass along any data you want to the Result object. Examples might include passing a DBI
 handle so your object could query a database directly based on some method you define.
 
+
+=item serial_format
+
+What format should serialized Perl values be assumed to be? The default is C<yaml>.
+You might also specify C<json>. If you have serialized values in some other format,
+then you'll need to subclass SWISH::API::Object::Result and override deserialize().
+
+See L<SWISH::Prog::Object>.
+
 =back
 
 
@@ -226,7 +270,6 @@ Get/set the I<properties> hash ref passed in new().
 Utitlity method. Returns sorted array of property names. Shortcut for:
 
  sort keys %{ $swish->properties }
- 
 
 
 =head1 SWISH::API::Object::Result
@@ -235,6 +278,11 @@ The internal SWISH::API::Object::Result class is used to extend the SWISH::API
 next_result() method with a next_result_after() method. See SWISH::API::More for
 documentation about how the *_after() methods work.
 
+=head2 deserialize( I<format>, I<prop_val> )
+
+Called for each property value. The I<format> deserialize() expects is based
+on C<serial_format> in SWISH::API::Object->new().
+
 
 =head1 SEE ALSO
 
